#!/usr/bin/python3
import argparse
import configparser
import os

from pytest_config import (
    update_confg_version,
    pretty
)

HOME = os.environ['HOME']
TEMPLATES_REL_DIR = '.pytest_config/templates'
src = lambda f: os.path.join(HOME, TEMPLATES_REL_DIR, f)

PROJECT_DIR = os.environ['PWD']
dest = lambda f: os.path.join(PROJECT_DIR, f)

TEMPLATE_PATHS = {
    'pytest.ini': {'src': src('pytest.ini'), 'dest': dest('pytest.ini')},
    '.coveragerc': {'src': src('coveragerc'), 'dest': dest('.coveragerc')}
}

###############################################################################
# Define command line arguments
###############################################################################
desc = 'Keep py.test configuration files for this project up-to-date.'
parser = argparse.ArgumentParser(description=desc)

parser.add_argument('-y',
                    dest='overwrite', action='store_true', default=False,
                    help='Do not prompt for anything. Assumes you want '
                    'all the default settings.')
parser.add_argument('-p', '--fix-path',
                    dest='fix_path', action='store_true', default=False,
                    help='Only fix the PYTHONPATH for pytest-django and '
                    'finish without updating. This occurs by default whenever '
                    'you update your configuration files.')
parser.add_argument('--coverage',
                    dest='update_coveragerc', action='store_true',
                    default=False, help='Update .coveragerc file excluding '
                    'other files not explicitly requested.')
parser.add_argument('--pytest-ini',
                    dest='update_pytestini', action='store_true',
                    default=False, help='Update pytest.ini file excluding '
                    'other files not explicitly requested.')

args = parser.parse_args()


###############################################################################
# Updater functions
###############################################################################
def write_to_file(template, config):
    """
    Writes the given configurations into the given file name.
    After writting out, updates the file version to the current version.

    :param str name: The name of the template to write out.
    :param ConfigParser.ConfigParser config: The configurations to write.

    """
    with open(TEMPLATE_PATHS[template]['dest'], 'w') as current:
        config.write(current)
    update_confg_version(TEMPLATE_PATHS, template)


def update_config_file_defaults(template):
    """
    Updates a pytest_config configuration file by interactively merging
    the defaults into the current, customized project configuration.

    :param str template: The key to a template in TEMPLATE_PATHS global

    """
    #  Read the default configurations template
    default_config = configparser.ConfigParser()
    try:
        with open(TEMPLATE_PATHS[template]['src']) as defaults:
            default_config.readfp(defaults)
    except IOError:
        pretty.print_error('ERROR: Unable to update. The template for', template,
                     'is missing. Please check that or reinstall pytest_config'
                     'to fix this.')
        raise SystemExit(1)

    # Read the current configurations file
    current_config = configparser.ConfigParser()
    try:
        with open(TEMPLATE_PATHS[template]['dest']) as current:
            current_config.readfp(current)
            short_name = current.name.rsplit('/', 1)[-1]
    except IOError:
        pretty.print_error('ERROR: Unable to update. Missing local', template,
                     'file. Please run `pytest_config.init` first')
        raise SystemExit(1)

    pretty.print_warning('Updating ', short_name, '\b...', new_line=False)

    # Check if an update is necessary
    up_to_date = False
    for section in default_config.sections():
        up_to_date = default_config.items(section) == current_config.items(section)
        if not up_to_date:
            break

    if up_to_date: # Everything is up-to-date, exit
        pretty.print_success(' [UP TO DATE]')
        update_confg_version(TEMPLATE_PATHS, template)
        return

    # Changes detected, perform interactive update,
    # but first, we need to know if the user wants to be prompted.
    if args.overwrite:
        # Automaitc overwrite requested. Set all the default values.
        pretty.print_success(' [DONE]')
        write_to_file(template, default_config)

    else:
        for section in default_config.sections():
            pretty.print_color(['\nSection:', [section]], bold=True, new_line=False)
            if default_config.items(section) == current_config.items(section):
                # Section up-to-date, move to the next
                pretty.print_success(' [UP TO DATE]\n', new_line=False)
                continue
            print()  # just print a new line, for prettyfying the output

            for name, default in default_config.items(section):
                # ask if they want to keep it or use the default
                if current_config.has_option(section, name) and \
                    current_config.get(section, name) != default:
                    # @TODO: Read and compare multiple values in the same line
                    # and ask whether to keep/replace individual differences.
                    pretty.print_color(['*%s:' % name], bold=True)
                    print(pretty.colorize_text('  default: %s' % default,
                                                color=pretty.RED))
                    print(pretty.colorize_text(
                        '  current: %s' % current_config.get(section, name),
                        color=pretty.GREEN))
                    prompt_msg = '  Keep the current value? [y/n] (default=n) > '
                    keep_current = input(prompt_msg).lower()
                    if keep_current == 'y':
                        continue
                current_config.set(section, name, default)
        write_to_file(template, current_config)

def main():
    # update only the requested file(s)
    if any([args.update_coveragerc, args.update_pytestini]):
        if args.update_pytestini:
            update_config_file_defaults('pytest.ini')
        elif args.update_coveragerc:
            update_config_file_defaults('.coveragerc')

    else:  # update all files
        update_config_file_defaults('pytest.ini')
        update_config_file_defaults('.coveragerc')



if __name__ == '__main__':
    main()
else:
    parser.print_help()
